@@ -3,7 +3,7 @@
  *
  * Userspace RCU library - test program (with baatch reclamation)
  *
- * Copyright February 2009 - Mathieu Desnoyers <mathieu.desnoyers@polymtl.ca>
+ * Copyright February 2009 - Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -86,6 +86,9 @@ static unsigned long duration;
 /* read-side C.S. duration, in loops */
 static unsigned long rduration;
 
+/* write-side C.S. duration, in loops */
+static unsigned long wduration;
+
 static inline void loop_sleep(unsigned long l)
 {
 	while(l-- != 0)
@@ -285,6 +288,8 @@ void *thr_writer(void *data)
 		new->a = 8;
 		old = rcu_xchg_pointer(&test_rcu_pointer, new);
 #endif
+		if (unlikely(wduration))
+			loop_sleep(wduration);
 		rcu_gc_reclaim(wtidx, old);
 		nr_writes++;
 		if (unlikely(!test_duration_write()))
@@ -307,6 +312,7 @@ void show_usage(int argc, char **argv)
 #endif
 	printf(" [-d delay] (writer period (us))");
 	printf(" [-c duration] (reader C.S. duration (in loops))");
+	printf(" [-e duration] (writer C.S. duration (in loops))");
 	printf(" [-v] (verbose output)");
 	printf(" [-a cpu#] [-a cpu#]... (affinity)");
 	printf("\n");
@@ -387,6 +393,13 @@ int main(int argc, char **argv)
 			}
 			wdelay = atol(argv[++i]);
 			break;
+		case 'e':
+			if (argc < i + 2) {
+				show_usage(argc, argv);
+				return -1;
+			}
+			wduration = atol(argv[++i]);
+			break;
 		case 'v':
 			verbose_mode = 1;
 			break;
@@ -455,11 +468,11 @@ int main(int argc, char **argv)
 	
 	printf_verbose("total number of reads : %llu, writes %llu\n", tot_reads,
 	       tot_writes);
-	printf("SUMMARY %-25s testdur %4lu nr_readers %3u rdur %6lu "
+	printf("SUMMARY %-25s testdur %4lu nr_readers %3u rdur %6lu wdur %6lu "
 		"nr_writers %3u "
 		"wdelay %6lu nr_reads %12llu nr_writes %12llu nr_ops %12llu "
 		"batch %u\n",
-		argv[0], duration, nr_readers, rduration,
+		argv[0], duration, nr_readers, rduration, wduration,
 		nr_writers, wdelay, tot_reads, tot_writes,
 		tot_reads + tot_writes, reclaim_batch);
 	free(tid_reader);
