@@ -6,7 +6,7 @@
  *
  * System definitions.
  *
- * Copyright (c) 2009 Mathieu Desnoyers <mathieu.desnoyers@polymtl.ca>
+ * Copyright (c) 2009 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
  *
  * THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY EXPRESSED
  * OR IMPLIED.  ANY USE IS AT YOUR OWN RISK.
@@ -46,9 +46,9 @@
  */
 #define STORE_SHARED(x, v)		\
 	({				\
-		_STORE_SHARED(x, v);	\
+		typeof(x) _v = _STORE_SHARED(x, v);	\
 		smp_wmc();		\
-		(v);			\
+		_v;			\
 	})
 
 #endif /* _URCU_SYSTEM_H */
